#include "kwlantraywin.h"
#include "wlanlib.h"
#include "kwlaninterface.h"
#include "kdialup.h"
#include "kstartsupplicantdlg.h"

#include <kstdaction.h>
#include <kpopupmenu.h>
#include <kaction.h>
#include <qtooltip.h>
#include <kiconloader.h>
#include <qpixmap.h>
#include <klocale.h>
#include <kdebug.h>






KWlanTrayWin::KWlanTrayWin(QWidget *parent, const char *name)
    : KSystemTray(parent, name)
{
    int id;
    m_interface = 0L;
    m_ifname="";
    m_interfaceUp = FALSE;
    //actionCollection()->remove( actionCollection()->action( "quit" ) );
    m_quit = KStdAction::quit( this, SIGNAL( quitSelected() ), actionCollection() );
    m_about = KStdAction::aboutApp( this, SIGNAL( aboutSelected() ), actionCollection() );
    m_interfaceEnableMenu = new KAction(this,"interfaceEnale");
    connect (m_interfaceEnableMenu,SIGNAL(activated()),this, SLOT(interfaceEnable()));

    setPixmap(loadIcon("kwlan/disconnected"));
    m_tooltip = new KWlanToolTip(this);
    m_profiles.setAutoDelete(TRUE);
    m_profileMenu = new KPopupMenu();
    connect (m_profileMenu, SIGNAL(activated(int )),SLOT(selectProfile(int )));
    m_wpaMenu = new KPopupMenu();
    m_wpaMenu->insertItem(i18n("Start WPA Supplicant"),WPASTARTID);
    m_wpaMenu->setItemParameter(WPASTARTID,WPASTARTID);
    m_wpaMenu->insertItem(i18n("Stop WPA Supplicant"),WPASTOPID);
    m_wpaMenu->setItemParameter(WPASTOPID,WPASTOPID);
    m_wpaMenu->insertItem(i18n("Log Viewer"),WPALOGID);
    m_wpaMenu->setItemParameter(WPALOGID,WPALOGID);
    connect (m_wpaMenu,SIGNAL(activated(int )), this, SLOT(handleWpaMenuEvent(int )));
    m_wpaMenu->setItemEnabled(WPASTOPID,FALSE);
    
    m_timer = new QTimer(this);
    if (m_timer){
        connect (m_timer, SIGNAL(timeout()),SLOT(updateIcon()));
        m_timer->start(2000,FALSE);
    }
    m_configuration = 0L;
}

KWlanTrayWin::~KWlanTrayWin() {
    if (m_timer){
        m_timer->stop();
        delete (m_timer);
    } 
    delete m_tooltip;
    m_tooltip = 0;
}

void KWlanTrayWin::setInterface(KWlanInterface *interface)
{
    if (!interface) return;
    m_interface = interface;
    connect (m_interface, SIGNAL(interfaceUp( bool )), this, SLOT (setInterfaceEnableMenu( bool )));
    connect (this, SIGNAL(interfaceEnable( bool )), m_interface, SLOT (enableInterface(bool)));
    connect (m_interface, SIGNAL(interfaceGone(QString)), this, SLOT (slotInterfaceGone(QString )));
    connect (this, SIGNAL(viewLog()),m_interface,SLOT(logViewer()));
    connect (this, SIGNAL(editProfiles()), m_interface,SLOT(editProfiles()));
    connect (this, SIGNAL(scan()), m_interface, SLOT(slotScan()));
    connect (this, SIGNAL(profileChanged(QString)), m_interface, SLOT(profileActivate(QString)));
    connect (m_interface, SIGNAL(wpaStatusChanged(bool)), this, SLOT(enableStartWpaMenu(bool)));
    m_confFile = QString("kwlanrc.") + interface->getInterfaceName();
    m_configuration = new Configuration(m_confFile);
    updateNetworks();

}

 
void KWlanTrayWin::contextMenuAboutToShow( KPopupMenu *r ) {
    QString *prof, curProf;
    updateNetworks();
    if (! m_interface) return;
    curProf = m_interface->getCurrentProfile();
    //kdDebug() << "Current profile" << curProf << endl;
    int id;
    r->clear();
    r->insertTitle(SmallIcon ("kwlan"), "KWlan - KDE Connection Manager");
    // Now fill int the profiles into profile menu
    id = 1;
    m_profileMenu->clear();
    //if (m_interfaceUp && (m_interface->getType()!=KWlanInterface::PPP)){
    if (m_interface->getType()!=KWlanInterface::PPP){
            m_profileMenu->insertItem(i18n("Edit profiles"),id);
        m_profileMenu->setItemParameter(id,id);
        id++;
        m_profileMenu->insertItem(i18n("Scan"),id);
        m_profileMenu->setItemParameter(id,id);
        id++;
        if (!m_profiles.isEmpty())
        {
            m_profileMenu->insertSeparator(id);
            id++;
        }
        prof = m_profiles.first();
        while (prof){
            //kdDebug() << "Inserting profile " << *prof << endl;
            m_profileMenu->insertItem(*prof,id);
            m_profileMenu->setItemParameter(id,id);
            if (*prof == curProf)
                m_profileMenu->setItemChecked(id, TRUE);
            else 
                m_profileMenu->setItemChecked(id, FALSE);
            prof=m_profiles.next();
            id++;
        }
        r->insertItem(i18n("Profiles"),m_profileMenu);
        r->insertItem(i18n("Wpa"),m_wpaMenu);
    }
    //now set the interface enable / disable menu
    QString enableMenu;
    if (m_interfaceUp) enableMenu = i18n("Disable");
    else enableMenu = i18n("Enable");
    m_interfaceEnableMenu->setText(enableMenu);
    m_interfaceEnableMenu->plug(r);
    if (m_interface->getType() != KWlanInterface::PPP) 
    {
        m_about->plug( r );
        m_quit->plug( r );
    }
    
}

void KWlanTrayWin::clearProfiles()
{
    while (!m_profiles.isEmpty()) m_profiles.removeFirst();
}

void KWlanTrayWin::updateNetworks()
{
    QStringList networks;
    clearProfiles();
    if (!m_interface) return;
    networks= m_interface->listProfiles();
    for (QStringList::Iterator it=networks.begin();it != networks.end();it++){
        m_profiles.append(new QString(*it));
    }
}

void KWlanTrayWin::setInterfaceEnableMenu( bool up)
{
    if (!m_interface) return;
    m_interfaceUp = up;
}
            
void KWlanTrayWin::interfaceEnable()
{
    if (!m_interface) return;
    emit (interfaceEnable(!m_interfaceUp));
}
const QString & KWlanTrayWin::getInterfaceName() const {
    return m_ifname;
}
        


void KWlanTrayWin::updateIcon()
{
    //QPixmap trayIcon;
    // Do we have a wireless interface?
    if (!m_interface ) return;
    interfaceData data = m_interface->getInterfaceData();
    wirelessData wData = m_interface->getWirelessData();
    if (m_interface->getType() == KWlanInterface::PPP)
    {
        setPixmap(loadIcon("kwlan/dialup"));
    }
    else {
        if (data.wirelessDevice) {
            if (!data.connected )
                setPixmap(loadIcon("kwlan/disconnected"));
            else if (!m_ifname.isEmpty()){
                
                //iwLib iw(m_ifname);
                
                //if (iw.getQuality() >35) 
                int index = wData.linkQuality.find('/');
                if (index != -1)
                    index = wData.linkQuality.left(index).toInt();
                else index = 100;

                if (index > 35)
                    setPixmap(loadIcon("kwlan/connected"));
                else setPixmap(loadIcon("kwlan/weakconnection"));
                
            }
        }
        else {
        // wired interface
            if (!data.connected)
                setPixmap(loadIcon("kwlan/wired-disconnected"));
            else setPixmap(loadIcon("kwlan/wired-connected"));
        }
    }
}


void KWlanTrayWin::setInterfaceName(const QString &ifname)
{
    m_ifname = ifname;
}

void KWlanTrayWin::setIpAddress(const QString &ipaddress){
    m_ipaddress = ipaddress;
}


void KWlanTrayWin::selectProfile(int id)
{
    //kdDebug() <<"select Profile " <<m_profileMenu->text(id) << endl;
    switch (id)
    {
        case 1:
            emit (editProfiles());
            break;
        case 2:
            emit(scan());
            break;
        default:
            emit profileChanged(m_profileMenu->text(id));
    }
}

void KWlanTrayWin::wpaStart()
{
    QString driver;
    if (!m_interface) return;
    if (m_interface->getInterfaceData().wirelessDevice)
    {
        KStartSupplicantDlg *startSupplicant = new KStartSupplicantDlg();
        if (!startSupplicant) return;
        if (!m_configuration) return;
        m_configuration->readDriver(driver);
        startSupplicant->setData( & m_interface->getInterfaceName() ,& driver);
        startSupplicant->show();
        startSupplicant->exec();
        if (startSupplicant->m_start){
            m_interface->startWpa(startSupplicant->getDriver());
        }
    }

    else {
        //wired device, so drvier is wired
        m_interface->startWpa(QString("wired"));
    }
}

void KWlanTrayWin::wpaStop()
{
    if (!m_interface) return;
    m_interface->stopWpa();
}
void KWlanTrayWin::handleWpaMenuEvent(int menu)
{
    switch (menu)
    {
        case WPASTARTID:
            wpaStart();
            break;
        case WPASTOPID:
            wpaStop();
            break;
        case WPALOGID:
            emit (viewLog());
            break;
    }
}

void KWlanTrayWin::enableStartWpaMenu(bool enable)
{
    m_wpaMenu->setItemEnabled(WPASTARTID,!enable);
    m_wpaMenu->setItemEnabled(WPASTOPID,enable);
    m_wpaMenu->setItemEnabled(WPALOGID,enable);
}

void KWlanTrayWin::slotInterfaceGone(QString interface)
{
    kdDebug() << "Closing traywin..." << endl;
    m_interface=0L;
    m_timer->stop();
    close();
    deleteLater();
}

void KWlanTrayWin::slotShowTrayWin(QString interface, bool showWin)
{
    if (interface != m_ifname) return;
    if (showWin) show();
    else hide();
}


KWlanToolTip::KWlanToolTip(QWidget *parent): QToolTip(parent) {
}
    
KWlanToolTip::~KWlanToolTip() {
    
}
        
void KWlanToolTip::maybeTip(const QPoint &p) {
    QString tooltip;
    KWlanTrayWin *parent = (KWlanTrayWin *)parentWidget();
        
    // We can't show tooltip if no interface was specified
    if (!parent->getInterface())
    {
        kdDebug() << "Now interface for tooltip found" << endl;
        return;
    }
    const QString &ifname = parent->getInterfaceName();
    interfaceData data = parent->getInterface()->getInterfaceData();
    QString network = parent->getProfile();
    if (data.connected) {
//        IwLib iw(ifname);
        tooltip  = QString("<table cellspacing=0 cellpadding=0 border=0>");
        if (network.isEmpty())
            tooltip += QString(i18n("<tr><th colspan=2 align=center>Interface %1</th></tr>")).arg(ifname);
        else 
            tooltip += QString(i18n("<tr><th colspan=2 align=center>Interface %1 - connected to %2</th></tr>")).arg(ifname,network);
        if (data.wirelessDevice) {
            wirelessData wData = parent->getInterface()->getWirelessData();
            tooltip += QString(i18n("<tr><td>Connected to: </td><td> <b>%1</b></td></tr>")).arg(wData.essid);
            tooltip += QString(i18n("<tr><td>Bitrate:</td><td> <b>%1 Mbit/s</b></td></tr>")).arg(wData.bitRate);    
            tooltip += QString(i18n("<tr><td>Link quality:</td><td> <b>%1</b></td></tr>")).arg(wData.linkQuality);
            tooltip += QString(i18n("<tr><td>Signal Level:</td><td> <b>%1</b></td></tr>")).arg(wData.signal);
            tooltip += QString(i18n("<tr><td>Frequency:</td><td> <b>%1</b></td></tr>")).arg(wData.frequency);
            tooltip += QString(i18n("<tr><td>Channel:</td><td> <b>%1</b></td></tr>")).arg(wData.channel);
        }
        tooltip += QString(i18n("<tr><td>IP address:</td><td> <b>%1</b></td></tr>")).arg(data.ipAddress);
        tooltip += QString(i18n("<tr><td>Netmask:</td><td> <b>%1</b></td></tr>")).arg(data.subnetMask);
        tooltip += QString(i18n("<tr><td>Broadcast address:</td><td> <b>%1</b></td></tr>")).arg(data.broadcastAddress);
        tooltip += QString(i18n("<tr><td>Bytes received:</td><td> <b>%1</b></td></tr>")).arg(data.rxString);
        tooltip += QString(i18n("<tr><td>Bytes sent:</td><td> <b>%1</b></td></tr>")).arg(data.txString);
        tooltip += QString(i18n("<tr><td>Packets received:</td><td> <b>%1</b></td></tr>")).arg(data.rxPackets);
        tooltip += QString(i18n("<tr><td>Packets sent:</td><td> <b>%1</b></td></tr>")).arg(data.txPackets);
        tooltip += QString("</table>");
    }
    else {
        tooltip  = QString(i18n("Interface : <b>%1</b><br />")).arg(ifname);
        tooltip += i18n("Disconnected");
    }
                 
    tip(parent->rect(), tooltip);
}

